@@ -10,7 +10,7 @@ use XS::Install::Deps;
 use XS::Install::Util;
 use XS::Install::Payload;
 
-our $VERSION = '1.2.15';
+our $VERSION = '1.2.17';
 my $THIS_MODULE = 'XS::Install';
 
 our @EXPORT_OK = qw/write_makefile not_available/;
@@ -314,18 +314,22 @@ sub process_binary {
     my %xsi;
     foreach my $xsfile (keys %{$params->{XS}}, map { _scan_files($xs_mask, $_) } @{$params->{SRC}}) {
         my $cfile = $params->{XS}{$xsfile};
+        my $suffix;
         unless ($cfile) {
             my $cext = $params->{CPLUS} ? 'cc' : 'c';
             if ($xsfile =~ /\.xs$/) {
                 $cfile = $xsfile;
                 $cfile =~ s/\.xs$/_xsgen.$cext/;
+                $suffix = "_xsgen.$cext";
             } else {
                 $cfile = "$xsfile.$cext";
             }
             $params->{XS}{$xsfile} = $cfile;
         }
-        my $suffix = $cfile;
-        $suffix =~ s/^[^.]+//;
+        unless ($suffix) {
+            $suffix = $cfile;
+            $suffix =~ s/^[^.]+//;
+        }
         
         my $xsi_deps = XS::Install::Deps::find_xsi_deps([$xsfile])->{$xsfile} || [];
         map { $xsi{$_} = $xsfile } @$xsi_deps;
@@ -401,14 +405,19 @@ sub process_CLIB {
             $build_cmd = "$make $info->{FLAGS} $info->{TARGET}";
             $clean_cmd = "$make clean";
         }
-        
-        my $static = $info->{FILE} =~ /\.l?a$/ ? 1 : 0;
-        my $path = $info->{DIR}.'/'.$info->{FILE};
-        $clibs .= "$path ";
+        $info->{FILE} = [$info->{FILE}] if exists $info->{FILE} && ref $info->{FILE} ne 'ARRAY';
+        my $path = '';
+        my $static = 0;
+        for my $f (@{$info->{FILE} || []}) {
+            next unless $f;
+            $static = 1 if $f =~ /\.l?a$/;
+            $path .= $info->{DIR}.'/'.$f.' ';
+        }
+        $clibs .= $path;
         
         push @{$params->{postamble}}, "$path : ; cd $info->{DIR} && $build_cmd\n";
         push @{$params->{postamble}}, "clean :: ; cd $info->{DIR} && $clean_cmd\n" if $clean_cmd;
-        if ($info->{FILE} =~ /\.l?a$/) {
+        if ($static) {
             push @{$params->{MODULE_INFO}{STATIC_LIBS}}, "$wa_open $path $wa_close";
         } else {
             push @{$params->{MODULE_INFO}{SHARED_LIBS}}, $path;
@@ -537,6 +546,11 @@ sub process_LD {
         $params->{MODULE_INFO}{SHARED_LIBS_LINKING} = $str;
         $params->{LDFROM} .= ' '.$str if $str;
     }
+    
+    if (my $ldf = $params->{LDDLFLAGS}) {
+        my $cf = $Config{lddlflags};
+        $params->{LDDLFLAGS} = string_merge($cf, $ldf) if index($ldf, $cf) == -1;
+    }
 }
 
 sub process_test {
@@ -567,6 +581,7 @@ sub process_test {
         CC        => $tp->{CC} || $params->{CC},
         LD        => $tp->{LD} || $params->{LD},
         LDFROM    => $ldfrom,
+        LDDLFLAGS => string_merge($params->{LDDLFLAGS}, $tp->{LDDLFLAGS}),
         INC       => string_merge($params->{MODULE_INFO}{ORIG}{INC}, $tp->{INC}),
         CCFLAGS   => string_merge($params->{MODULE_INFO}{ORIG}{CCFLAGS}, $tp->{CCFLAGS}),
         DEFINE    => string_merge($params->{DEFINE}, $tp->{DEFINE}),
@@ -579,7 +594,6 @@ sub process_test {
         NO_MYMETA => 1,
         _XSTEST   => 1,
     );
-
     
     my $mm_args = makemaker_args(%args);
     return undef unless has_binary($mm_args);
@@ -662,7 +676,8 @@ sub binary_module_version {
 sub has_c      { return $_[0]->{C} && scalar(@{$_[0]->{C}}) ? 1 : 0 }
 sub has_object { return $_[0]->{OBJECT} && scalar(@{$_[0]->{OBJECT}}) ? 1 : 0 }
 sub has_xs     { return $_[0]->{XS} && scalar(keys %{$_[0]->{XS}}) ? 1 : 0 }
-sub has_binary { return has_c($_[0]) || has_object($_[0]) || has_xs($_[0]) }
+sub has_ext    { return $_[0]->{MODULE_INFO}{STATIC_LIBS} && scalar(@{$_[0]->{MODULE_INFO}{STATIC_LIBS}}) ? 1 : 0 }
+sub has_binary { return has_c($_[0]) || has_object($_[0]) || has_xs($_[0]) || has_ext($_[0]) }
 
 sub merge_optimize {
     my $to = shift;
@@ -792,7 +807,11 @@ sub _string_merge {
 sub string_merge {
     my $s = shift;
     $s //= '';
-    $s .= ($_ // '') for @_;
+    for my $val (@_) {
+        next unless defined($val) && length($val);
+        if (!$s) { $s = $val }
+        else { $s .= " $val"; }
+    }
     return $s;
 }
 
@@ -811,6 +830,13 @@ sub c2obj_file {
     
     sub init_methods {
         no warnings 'redefine';
+
+        # change "subdirs-test*" rule type from "::" to ":" as GNU make has a bug and can't do its deps in parallel
+        sub _fix_subdirs {
+            my $s = shift;
+            $s =~ s/^((subdirs-test(?:_dynamic|_static)?\s*)+)::/$1:/mg;
+            return $s;
+        }
         
         *postamble = sub {
             my $self = shift;
@@ -824,8 +850,10 @@ sub c2obj_file {
                 ++$i;
             }
             
-            return join("\n\n", @list);
+            return _fix_subdirs(join("\n\n", @list));
         };
+
+        *test = sub { return _fix_subdirs(shift->SUPER::test(@_)) };
         
         if ($fix_bsd_make_j) { # bsd's make has a bug: wrong value of $* when building in parallel, we use $< instead
             *c_o = sub {
